import pkgutil

name = "gradio_agents"

with open(r"version.txt", "w", encoding='utf-8')as f:
    f.write('1.0.7')

import gradio_agents.components as components
import gradio_agents.inputs as inputs
import gradio_agents.outputs as outputs
import gradio_agents.processing_utils
import gradio_agents.templates
import gradio_agents.themes as themes
from gradio_agents.blocks import Blocks
from gradio_agents.components import (
    HTML,
    JSON,
    AnnotatedImage,
    Annotatedimage,
    Audio,
    BarPlot,
    Button,
    Carousel,
    Chatbot,
    Checkbox,
    CheckboxGroup,
    Checkboxgroup,
    Code,
    ColorPicker,
    DataFrame,
    Dataframe,
    Dataset,
    Dropdown,
    File,
    Gallery,
    Highlight,
    HighlightedText,
    Highlightedtext,
    Image,
    Interpretation,
    Json,
    Label,
    LinePlot,
    Markdown,
    Model3D,
    Number,
    Plot,
    Radio,
    ScatterPlot,
    Slider,
    State,
    StatusTracker,
    Text,
    Textbox,
    Spark,
    TimeSeries,
    Timeseries,
    UploadButton,
    Variable,
    Video,
    component,
)
from gradio_agents.events import SelectData
from gradio_agents.exceptions import Error
from gradio_agents.external import load
from gradio_agents.flagging import (
    CSVLogger,
    FlaggingCallback,
    HuggingFaceDatasetJSONSaver,
    HuggingFaceDatasetSaver,
    SimpleCSVLogger,
)
from gradio_agents.helpers import EventData, Progress, make_waveform, skip, update
from gradio_agents.helpers import create_examples as Examples  # noqa: N812
from gradio_agents.interface import Interface, TabbedInterface, close_all
from gradio_agents.ipython_ext import load_ipython_extension
from gradio_agents.layouts import Accordion, Box, Column, Group, Row, Tab, TabItem, Tabs, Floating
from gradio_agents.mix import Parallel, Series
from gradio_agents.routes import Request, mount_gradio_app
from gradio_agents.templates import (
    Files,
    ImageMask,
    ImagePaint,
    List,
    Matrix,
    Mic,
    Microphone,
    Numpy,
    Paint,
    Pil,
    PlayableVideo,
    Sketchpad,
    TextArea,
    Webcam,
)
from gradio_agents.themes import Base as Theme

current_pkg_version = (
    (pkgutil.get_data(__name__, "version.txt") or b"").decode("ascii").strip()
)
__version__ = current_pkg_version
