README_CONTENT = """
---
tags: [gradio-theme]
title: {theme_name}
colorFrom: orange
colorTo: purple
sdk: gradio
sdk_version: {gradio_version}
app_file: app.py
pinned: false
license: apache-2.0
---
# {theme_name}
## Description
{description}
## Contributions
Thanks to [@{author}](https://huggingface.co/{author}) for adding this gradio theme!
"""
